@@ -1,5 +1,5 @@
 *
-* $Id: pbe96.f 26893 2015-02-26 16:19:02Z bylaska $
+* $Id$
 *
 
 *    ************************************
@@ -53,8 +53,8 @@ c     ***** PBE96 GGA exchange constants ******
 c     ****** PBE96 GGA correlation constants ******
       real*8 GAMMA,BETA,BOG
       parameter (GAMMA	= 0.031090690869655d0)
-      !parameter (BETA	= 0.066724550603149d0)
-      parameter (BETA	= 0.066725d0)
+      parameter (BETA	= 0.066724550603149d0)
+      !parameter (BETA	= 0.066725d0)
       parameter (BOG    = BETA/GAMMA)
 
 
@@ -154,6 +154,7 @@ c     **** local variables ****
       fdnx_const = -3.0d0/(8.0d0*pi)
 
       
+!$OMP DO
       do i=1,n2ft3d
          nup     = dn_in(i,1)+ETA
          agrup   = agr_in(i,1)
@@ -358,6 +359,7 @@ c        ********************************************
          fdn(i,3) = c_parameter*t_agr*Hpbe_t
 
       end do
+!$OMP END DO
       
       
       
@@ -471,6 +473,7 @@ c     **** local variables ****
       rs_scale   = (0.75d0/pi)**onethird
       fdnx_const = -3.0d0/(8.0d0*pi)
       
+!$OMP DO
       do i=1,n2ft3d
          n     = rho_in(i)+ETA
          agr   = agr_in(i)
@@ -508,7 +511,7 @@ c        **** calculate rs and t ****
 
          kf = (3.0d0*pi*pi*n)**onethird
          ks = dsqrt(4.0d0*kf/pi)
-         t  = agr/(2.0*ks*n)
+         t  = agr/(2.0d0*ks*n)
 
 
 c        **** unpolarized LDA correlation energy ****
@@ -553,6 +556,7 @@ c       write(*,*) "pbe96:",i,ec,fnc,fdnc
 
 
       end do
+!$OMP END DO
 
       return
       end
