@@ -45,7 +45,7 @@
  190   continue
        if(j.eq.14)buf=-buf
        goto 210
- 200   buf=buf+(dfloat(j-1)*fact)
+ 200   buf=buf+(dble(j-1)*fact)
        fact=fact*10.0d+0
  210   nstart=nstart-1
       enddo
@@ -56,4 +56,4 @@
       return
 
       END
-c $Id: extrct_real.f 22502 2012-05-20 06:11:57Z d3y133 $
+c $Id$
