@@ -748,6 +748,13 @@ int PARMCI_Malloc(void *ptr_arr[], armci_size_t bytes)
     return(0);
 }
 
+/*\
+ *  Wrapper on PARMCI_Malloc to keep old code from breaking
+\*/
+int PARMCI_Malloc_memdev(void *ptr_arr[], armci_size_t bytes, const char *device)
+{
+  return PARMCI_Malloc(ptr_arr,bytes);
+}
 
 
 /*\ shared memory is released to kr_malloc only on process 0
@@ -792,6 +799,11 @@ int PARMCI_Free(void *ptr)
     return 0;
 }
 
+int PARMCI_Free_memdev(void *ptr)
+{
+  return PARMCI_Free(ptr);
+}
+
 
 int ARMCI_Uses_shm()
 {
@@ -885,6 +897,15 @@ int ARMCI_Malloc_group(void *ptr_arr[], armci_size_t bytes,
     return(0);
 }
 
+/*\
+ *  Wrapper on ARMCI_Malloc_group to keep old code from breaking
+\*/
+int ARMCI_Malloc_group_memdev(void *ptr_arr[], armci_size_t bytes,
+                       ARMCI_Group *group, const char *device)
+{
+  return ARMCI_Malloc_group(ptr_arr,bytes,group);
+}
+
 
 /*\ shared memory is released to kr_malloc only on process 0
  *  with data server malloc cannot be used
