@@ -3,12 +3,30 @@
 #endif
 
 /* C and/or system headers */
+
+#if HAVE_STDIO_H
 #include <stdio.h>
+#endif
+
+#if HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+
+#if HAVE_ASSERT_H
 #include <assert.h>
+#endif
+
+#if HAVE_STRING_H
 #include <string.h>
+#endif
+
+#if HAVE_STRINGS_H
 #include <strings.h>
+#endif
+
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 /* 3rd party headers */
 #include <mpi.h>
@@ -795,7 +813,7 @@ int comex_puts(
         src_idx = 0;
         dst_idx = 0;
         for(j=1; j<=stride_levels; j++) {
-            src_idx += src_bvalue[j] * src_stride_ar[j-1];
+	  src_idx += (long) src_bvalue[j] * (long) src_stride_ar[j-1];
             if((i+1) % src_bunit[j] == 0) {
                 src_bvalue[j]++;
             }
@@ -805,7 +823,7 @@ int comex_puts(
         }
 
         for(j=1; j<=stride_levels; j++) {
-            dst_idx += dst_bvalue[j] * dst_stride_ar[j-1];
+	  dst_idx += (long) dst_bvalue[j] * (long) dst_stride_ar[j-1];
             if((i+1) % dst_bunit[j] == 0) {
                 dst_bvalue[j]++;
             }
@@ -868,7 +886,7 @@ int comex_gets(
     for(i=0; i<n1dim; i++) {
         src_idx = 0;
         for(j=1; j<=stride_levels; j++) {
-            src_idx += src_bvalue[j] * src_stride_ar[j-1];
+	  src_idx += (long) src_bvalue[j] * (long) src_stride_ar[j-1];
             if((i+1) % src_bunit[j] == 0) {
                 src_bvalue[j]++;
             }
@@ -880,7 +898,7 @@ int comex_gets(
         dst_idx = 0;
         
         for(j=1; j<=stride_levels; j++) {
-            dst_idx += dst_bvalue[j] * dst_stride_ar[j-1];
+	  dst_idx += (long) dst_bvalue[j] * (long) dst_stride_ar[j-1];
             if((i+1) % dst_bunit[j] == 0) {
                 dst_bvalue[j]++;
             }
@@ -958,7 +976,7 @@ int comex_accs(
         src_idx = 0;
         dst_idx = 0;
         for(j=1; j<=stride_levels; j++) {
-            src_idx += src_bvalue[j] * src_stride_ar[j-1];
+	  src_idx += (long) src_bvalue[j] * (long) src_stride_ar[j-1];
             if((i+1) % src_bunit[j] == 0) {
                 src_bvalue[j]++;
             }
@@ -968,7 +986,7 @@ int comex_accs(
         }
 
         for(j=1; j<=stride_levels; j++) {
-            dst_idx += dst_bvalue[j] * dst_stride_ar[j-1];
+	  dst_idx += (long) dst_bvalue[j] * (long) dst_stride_ar[j-1];
             if((i+1) % dst_bunit[j] == 0) {
                 dst_bvalue[j]++;
             }
@@ -1125,6 +1143,7 @@ int comex_barrier(comex_group_t comex_group)
     char *response_received_mask; /* which procs did we get ack from */
     int request_received_count = 0;
     int response_received_count = 0;
+    int ring_proc;
 
 #if DEBUG
     printf("[%d] comex_barrier(%d)\n", l_state.rank, comex_group);
@@ -1158,7 +1177,6 @@ int comex_barrier(comex_group_t comex_group)
     /* send out barrier requests to all procs in default group */
 #define USE_RING 1
 #if USE_RING
-    int ring_proc;
     for (ring_proc=0; ring_proc<group_size; ++ring_proc)
 #else
     for (group_proc=0; group_proc<group_size; ++group_proc)
@@ -1311,7 +1329,7 @@ int comex_barrier(comex_group_t comex_group)
 
 void comex_error(char *msg, int code)
 {
-    fprintf(stderr, msg);
+    fprintf(stderr, "%s", msg);
     fprintf(stderr,"Received an Error in Communication\n");
     MPI_Abort(l_state.world_comm, code);
 }
@@ -1465,6 +1483,18 @@ int comex_nbget(
 }
 
 
+int comex_nbacc(
+        int datatype, void *scale,
+        void *src, void *dst, int bytes,
+        int proc, comex_group_t group,
+        comex_request_t *hdl)
+{
+    int rc;
+    rc = comex_acc(datatype, scale, src, dst, bytes, proc, group);
+    return rc;
+}
+
+
 int comex_wait_proc(int proc, comex_group_t group)
 {
     comex_wait_all(group);
@@ -2119,7 +2149,7 @@ int comex_malloc(void **ptrs, size_t size, comex_group_t group)
   
     /* exchange buffer address */
     comex_barrier(group); /* end ARMCI epoch, enter MPI epoch */
-    rc = MPI_Allgather(MPI_IN_PLACE, 0, MPI_LONG, ptrs, 1, MPI_LONG, comm);
+    rc = MPI_Allgather(MPI_IN_PLACE, 0, MPI_CHAR, ptrs, SIZEOF_VOIDP, MPI_CHAR, comm);
     assert(MPI_SUCCESS == rc);
 #if DEBUG
     {
@@ -2138,6 +2168,12 @@ int comex_malloc(void **ptrs, size_t size, comex_group_t group)
     return COMEX_SUCCESS;
 }
 
+int comex_malloc_mem_dev(void *ptrs[], size_t size, comex_group_t group,
+        const char* device)
+{
+    return comex_malloc(ptrs,size,group);
+}
+
 
 int comex_free(void *ptr, comex_group_t group)
 {
@@ -2160,3 +2196,7 @@ int comex_free(void *ptr, comex_group_t group)
     return COMEX_SUCCESS;
 }
 
+int comex_free_dev(void *ptr, comex_group_t group)
+{
+    return comex_free(ptr, group);
+}
