@@ -20,16 +20,16 @@
 #include "message.h"
 #include "globalp.h"
 #include "armci.h"
+#include "ga_iterator.h"
 #include "ga-papi.h"
 #include "ga-wapi.h"
+#include "base.h"
 
 #ifdef MSG_COMMS_MPI
 extern ARMCI_Group* ga_get_armci_group_(int);
 #endif
 
 /* work arrays used in all routines */
-static Integer dims[MAXDIM], ld[MAXDIM-1];
-static Integer lo[MAXDIM],hi[MAXDIM];
 static Integer one_arr[MAXDIM]={1,1,1,1,1,1,1};
 
 #define GET_ELEMS(ndim,lo,hi,ld,pelems){\
@@ -58,6 +58,10 @@ void pnga_zero(Integer g_a)
   Integer ndim, type, me, elems, p_handle;
   Integer num_blocks;
   void *ptr;
+  Integer _dims[MAXDIM];
+  Integer _ld[MAXDIM-1];
+  Integer _lo[MAXDIM];
+  Integer _hi[MAXDIM];
   /*register Integer i;*/
   int local_sync_begin,local_sync_end;
 
@@ -70,22 +74,21 @@ void pnga_zero(Integer g_a)
   me = pnga_pgroup_nodeid(p_handle);
 
   pnga_check_handle(g_a, "ga_zero");
-  GA_PUSH_NAME("ga_zero");
 
   num_blocks = pnga_total_blocks(g_a);
 
-  pnga_inquire(g_a, &type, &ndim, dims);
+  pnga_inquire(g_a, &type, &ndim, _dims);
   if (num_blocks < 0) {
-    pnga_distribution(g_a, me, lo, hi);
+    pnga_distribution(g_a, me, _lo, _hi);
 
-    if ( lo[0]> 0 ){ /* base index is 1: we get 0 if no elements stored on p */
+    if ( _lo[0]> 0 ){ /* base index is 1: we get 0 if no elements stored on p */
 
       if (pnga_has_ghosts(g_a)) {
-        pnga_zero_patch(g_a,lo,hi);
+        pnga_zero_patch(g_a,_lo,_hi);
         return;
       }
-      pnga_access_ptr(g_a, lo, hi, &ptr, ld);
-      GET_ELEMS(ndim,lo,hi,ld,&elems);
+      pnga_access_ptr(g_a, _lo, _hi, &ptr, _ld);
+      GET_ELEMS(ndim,_lo,_hi,_ld,&elems);
 
       /* switch (type){ */
 /*         int *ia; */
@@ -122,7 +125,7 @@ void pnga_zero(Integer g_a)
       memset(ptr, 0, GAsizeofM(type)*elems);
 
       /* release access to the data */
-      pnga_release_update(g_a, lo, hi);
+      pnga_release_update(g_a, _lo, _hi);
     } 
   } else {
     pnga_access_block_segment_ptr(g_a, me, &ptr, &elems);
@@ -164,76 +167,9 @@ void pnga_zero(Integer g_a)
     pnga_release_update_block_segment(g_a, me);
   }
   if(local_sync_end)pnga_pgroup_sync(p_handle);
-  GA_POP_NAME;
 }
 
 
-
-#if 0
-/*\ COPY ONE GLOBAL ARRAY INTO ANOTHER
-\*/
-static void snga_copy_old(Integer g_a, Integer g_b)
-{
-Integer  ndim, ndimb, type, typeb, me, elems=0, elemsb=0;
-Integer dimsb[MAXDIM];
-void *ptr_a, *ptr_b;
-
-   me = pnga_nodeid();
-
-   GA_PUSH_NAME("ga_copy");
-
-   if(g_a == g_b) pnga_error("arrays have to be different ", 0L);
-
-   pnga_inquire(g_a,  &type, &ndim, dims);
-   pnga_inquire(g_b,  &typeb, &ndimb, dimsb);
-
-   if(type != typeb) pnga_error("types not the same", g_b);
-
-   if(!pnga_compare_distr(g_a,g_b))
-
-      pnga_copy_patch("n",g_a, one_arr, dims, g_b, one_arr, dimsb);
-
-   else {
-
-     pnga_sync();
-
-     pnga_distribution(g_a, me, lo, hi);
-     if(lo[0]>0){
-        pnga_access_ptr(g_a, lo, hi, &ptr_a, ld);
-        if (pnga_has_ghosts(g_a)) {
-          GET_ELEMS_W_GHOSTS(ndim,lo,hi,ld,&elems);
-        } else {
-          GET_ELEMS(ndim,lo,hi,ld,&elems);
-        }
-     }
-
-     pnga_distribution(g_b, me, lo, hi);
-     if(lo[0]>0){
-        pnga_access_ptr(g_b, lo, hi, &ptr_b, ld);
-        if (pnga_has_ghosts(g_b)) {
-          GET_ELEMS_W_GHOSTS(ndim,lo,hi,ld,&elems);
-        } else {
-          GET_ELEMS(ndim,lo,hi,ld,&elems);
-        }
-     }
-  
-     if(elems!= elemsb)pnga_error("inconsistent number of elements",elems-elemsb);
-
-     if(elems>0){
-        ARMCI_Copy(ptr_a, ptr_b, (int)elems*GAsizeofM(type));
-        pnga_release(g_a,lo,hi);
-        pnga_release(g_b,lo,hi);
-     }
-
-     pnga_sync();
-   }
-
-   GA_POP_NAME;
-}
-#endif
-
-
-
 /*\ COPY ONE GLOBAL ARRAY INTO ANOTHER
 \*/
 #if HAVE_SYS_WEAK_ALIAS_PRAGMA
@@ -246,11 +182,15 @@ Integer dimsb[MAXDIM],i;
 Integer a_grp, b_grp, anproc, bnproc;
 Integer num_blocks_a, num_blocks_b;
 Integer blocks[MAXDIM], block_dims[MAXDIM];
-void *ptr_a, *ptr_b;
+char *ptr_a, *ptr_b;
 int local_sync_begin,local_sync_end,use_put;
+_iterator_hdl hdl;
 
-   GA_PUSH_NAME("ga_copy");
 
+   Integer _dims[MAXDIM];
+   Integer _ld[MAXDIM-1];
+   Integer _lo[MAXDIM];
+   Integer _hi[MAXDIM];
    local_sync_begin = _ga_sync_begin; local_sync_end = _ga_sync_end;
    _ga_sync_begin = 1; _ga_sync_end=1; /*remove any previous masking*/
    a_grp = pnga_get_pgroup(g_a);
@@ -281,13 +221,13 @@ int local_sync_begin,local_sync_end,use_put;
 
    if(g_a == g_b) pnga_error("arrays have to be different ", 0L);
 
-   pnga_inquire(g_a,  &type, &ndim, dims);
+   pnga_inquire(g_a,  &type, &ndim, _dims);
    pnga_inquire(g_b,  &typeb, &ndimb, dimsb);
 
    if(type != typeb) pnga_error("types not the same", g_b);
    if(ndim != ndimb) pnga_error("dimensions not the same", ndimb);
 
-   for(i=0; i< ndim; i++)if(dims[i]!=dimsb[i]) 
+   for(i=0; i< ndim; i++)if(_dims[i]!=dimsb[i]) 
                           pnga_error("dimensions not the same",i);
 
    if ((pnga_is_mirrored(g_a) && pnga_is_mirrored(g_b)) ||
@@ -296,19 +236,20 @@ int local_sync_begin,local_sync_end,use_put;
         Copy operation is straightforward */
 
      if (use_put) {
+#if 0
        if (num_blocks_a < 0) {
-         pnga_distribution(g_a, me_a, lo, hi);
-         if(lo[0]>0){
-           pnga_access_ptr(g_a, lo, hi, &ptr_a, ld);
-           pnga_put(g_b, lo, hi, ptr_a, ld);
+         pnga_distribution(g_a, me_a, _lo, _hi);
+         if(_lo[0]>0){
+           pnga_access_ptr(g_a, _lo, _hi, &ptr_a, _ld);
+           pnga_put(g_b, _lo, _hi, ptr_a, _ld);
          }
        } else {
          if (!pnga_uses_proc_grid(g_a)) {
            for (i=me_a; i<num_blocks_a; i += anproc) {
-             pnga_distribution(g_a, i, lo, hi);
-             if (lo[0]>0) {
-               pnga_access_block_ptr(g_a, i, &ptr_a, ld);
-               pnga_put(g_b, lo, hi, ptr_a, ld);
+             pnga_distribution(g_a, i, _lo, _hi);
+             if (_lo[0]>0) {
+               pnga_access_block_ptr(g_a, i, &ptr_a, _ld);
+               pnga_put(g_b, _lo, _hi, ptr_a, _ld);
              }
            }
          } else {
@@ -322,14 +263,14 @@ int local_sync_begin,local_sync_end,use_put;
              /* find bounding coordinates of block */
              chk = 1;
              for (i = 0; i < ndim; i++) {
-               lo[i] = index[i]*block_dims[i]+1;
-               hi[i] = (index[i] + 1)*block_dims[i];
-               if (hi[i] > dims[i]) hi[i] = dims[i];
-               if (hi[i] < lo[i]) chk = 0;
+               _lo[i] = index[i]*block_dims[i]+1;
+               _hi[i] = (index[i] + 1)*block_dims[i];
+               if (_hi[i] > _dims[i]) _hi[i] = _dims[i];
+               if (_hi[i] < _lo[i]) chk = 0;
              }
              if (chk) {
-               pnga_access_block_grid_ptr(g_a, index, &ptr_a, ld);
-               pnga_put(g_b, lo, hi, ptr_a, ld);
+               pnga_access_block_grid_ptr(g_a, index, &ptr_a, _ld);
+               pnga_put(g_b, _lo, _hi, ptr_a, _ld);
              }
              /* increment index to get next block on processor */
              index[0] += topology[0];
@@ -342,20 +283,27 @@ int local_sync_begin,local_sync_end,use_put;
            }
          }
        }
+#else
+       pnga_local_iterator_init(g_a, &hdl);
+       while (pnga_local_iterator_next(&hdl,_lo,_hi,&ptr_a,_ld)) {
+         pnga_put(g_b, _lo, _hi, ptr_a, _ld);
+       }
+#endif
      } else {
+#if 0
        if (num_blocks_b < 0) {
-         pnga_distribution(g_b, me_b, lo, hi);
-         if(lo[0]>0){
-           pnga_access_ptr(g_b, lo, hi, &ptr_b, ld);
-           pnga_get(g_a, lo, hi, ptr_b, ld);
+         pnga_distribution(g_b, me_b, _lo, _hi);
+         if(_lo[0]>0){
+           pnga_access_ptr(g_b, _lo, _hi, &ptr_b, _ld);
+           pnga_get(g_a, _lo, _hi, ptr_b, _ld);
          }
        } else {
          if (!pnga_uses_proc_grid(g_a)) {
            for (i=me_b; i<num_blocks_b; i += bnproc) {
-             pnga_distribution(g_b, i, lo, hi);
-             if (lo[0]>0) {
-               pnga_access_block_ptr(g_b, i, &ptr_b, ld);
-               pnga_get(g_a, lo, hi, ptr_b, ld);
+             pnga_distribution(g_b, i, _lo, _hi);
+             if (_lo[0]>0) {
+               pnga_access_block_ptr(g_b, i, &ptr_b, _ld);
+               pnga_get(g_a, _lo, _hi, ptr_b, _ld);
              }
            }
          } else {
@@ -369,14 +317,14 @@ int local_sync_begin,local_sync_end,use_put;
              /* find bounding coordinates of block */
              chk = 1;
              for (i = 0; i < ndim; i++) {
-               lo[i] = index[i]*block_dims[i]+1;
-               hi[i] = (index[i] + 1)*block_dims[i];
-               if (hi[i] > dims[i]) hi[i] = dims[i];
-               if (hi[i] < lo[i]) chk = 0;
+               _lo[i] = index[i]*block_dims[i]+1;
+               _hi[i] = (index[i] + 1)*block_dims[i];
+               if (_hi[i] > _dims[i]) _hi[i] = _dims[i];
+               if (_hi[i] < _lo[i]) chk = 0;
              }
              if (chk) {
-               pnga_access_block_grid_ptr(g_b, index, &ptr_b, ld);
-               pnga_get(g_a, lo, hi, ptr_b, ld);
+               pnga_access_block_grid_ptr(g_b, index, &ptr_b, _ld);
+               pnga_get(g_a, _lo, _hi, ptr_b, _ld);
              }
              /* increment index to get next block on processor */
              index[0] += topology[0];
@@ -389,25 +337,31 @@ int local_sync_begin,local_sync_end,use_put;
            }
          }
        }
+#else
+       pnga_local_iterator_init(g_b, &hdl);
+       while (pnga_local_iterator_next(&hdl,_lo,_hi,&ptr_b,_ld)) {
+         pnga_get(g_a, _lo, _hi, ptr_b, _ld);
+       }
+#endif
      }
    } else {
      /* One global array is mirrored and the other is not */
      if (pnga_is_mirrored(g_a)) {
        /* Source array is mirrored and destination
           array is distributed. Assume source array is consistent */
-       pnga_distribution(g_b, me_b, lo, hi);
-       if (lo[0]>0) {
-         pnga_access_ptr(g_b, lo, hi, &ptr_b, ld);
-         pnga_get(g_a, lo, hi, ptr_b, ld);
+       pnga_distribution(g_b, me_b, _lo, _hi);
+       if (_lo[0]>0) {
+         pnga_access_ptr(g_b, _lo, _hi, &ptr_b, _ld);
+         pnga_get(g_a, _lo, _hi, ptr_b, _ld);
        } 
      } else {
        /* source array is distributed and destination
           array is mirrored */
        pnga_zero(g_b);
-       pnga_distribution(g_a, me_a, lo, hi);
-       if (lo[0] > 0) {
-         pnga_access_ptr(g_a, lo, hi, &ptr_a, ld);
-         pnga_put(g_b, lo, hi, ptr_a, ld);
+       pnga_distribution(g_a, me_a, _lo, _hi);
+       if (_lo[0] > 0) {
+         pnga_access_ptr(g_a, _lo, _hi, &ptr_a, _ld);
+         pnga_put(g_b, _lo, _hi, ptr_a, _ld);
        }
        pnga_merge_mirrored(g_b);
      }
@@ -423,7 +377,6 @@ int local_sync_begin,local_sync_end,use_put;
        pnga_pgroup_sync(b_grp);
      }
    }
-   GA_POP_NAME;
 }
 
 
@@ -448,12 +401,15 @@ int alen=0;
 Integer a_grp=0, b_grp=0;
 Integer num_blocks_a=0, num_blocks_b=0;
 
+Integer _dims[MAXDIM];
+Integer _ld[MAXDIM-1];
+Integer _lo[MAXDIM];
+Integer _hi[MAXDIM];
 Integer andim, adims[MAXDIM];
 Integer bndim, bdims[MAXDIM];
 
    _ga_sync_begin = 1; _ga_sync_end=1; /*remove any previous masking*/
 
-   GA_PUSH_NAME("ga_dot");
    a_grp = pnga_get_pgroup(g_a);
    b_grp = pnga_get_pgroup(g_b);
    if (a_grp != b_grp)
@@ -468,7 +424,6 @@ Integer bndim, bdims[MAXDIM];
      pnga_inquire(g_b, &type, &bndim, bdims);
      pnga_dot_patch(g_a, "n", one_arr, adims, g_b, "n", one_arr, bdims,
          value);
-     GA_POP_NAME;
      return;
    }
 
@@ -481,20 +436,19 @@ Integer bndim, bdims[MAXDIM];
        pnga_dot_patch(g_a, "n", one_arr, adims, g_b, "n", one_arr, bdims,
                       value);
        
-       GA_POP_NAME;
        return;
    }
    
    pnga_pgroup_sync(a_grp);
-   pnga_inquire(g_a,  &type, &ndim, dims);
+   pnga_inquire(g_a,  &type, &ndim, _dims);
    if(type != Type) pnga_error("type not correct", g_a);
-   pnga_distribution(g_a, me, lo, hi);
-   if(lo[0]>0){
-      pnga_access_ptr(g_a, lo, hi, &ptr_a, ld);
+   pnga_distribution(g_a, me, _lo, _hi);
+   if(_lo[0]>0){
+      pnga_access_ptr(g_a, _lo, _hi, &ptr_a, _ld);
       if (pnga_has_ghosts(g_a)) {
-        GET_ELEMS_W_GHOSTS(ndim,lo,hi,ld,&elems);
+        GET_ELEMS_W_GHOSTS(ndim,_lo,_hi,_ld,&elems);
       } else {
-        GET_ELEMS(ndim,lo,hi,ld,&elems);
+        GET_ELEMS(ndim,_lo,_hi,_ld,&elems);
       }
    }
 
@@ -502,15 +456,15 @@ Integer bndim, bdims[MAXDIM];
      elemsb = elems;
      ptr_b = ptr_a;
    }else {  
-     pnga_inquire(g_b,  &type, &ndim, dims);
+     pnga_inquire(g_b,  &type, &ndim, _dims);
      if(type != Type) pnga_error("type not correct", g_b);
-     pnga_distribution(g_b, me, lo, hi);
-     if(lo[0]>0){
-        pnga_access_ptr(g_b, lo, hi, &ptr_b, ld);
+     pnga_distribution(g_b, me, _lo, _hi);
+     if(_lo[0]>0){
+        pnga_access_ptr(g_b, _lo, _hi, &ptr_b, _ld);
         if (pnga_has_ghosts(g_b)) {
-          GET_ELEMS_W_GHOSTS(ndim,lo,hi,ld,&elemsb);
+          GET_ELEMS_W_GHOSTS(ndim,_lo,_hi,_ld,&elemsb);
         } else {
-          GET_ELEMS(ndim,lo,hi,ld,&elemsb);
+          GET_ELEMS(ndim,_lo,_hi,_ld,&elemsb);
         }
      }
    }
@@ -600,8 +554,8 @@ Integer bndim, bdims[MAXDIM];
    
       /* release access to the data */
       if(elems>0){
-         pnga_release(g_a, lo, hi);
-         if(g_a != g_b)pnga_release(g_b, lo, hi);
+         pnga_release(g_a, _lo, _hi);
+         if(g_a != g_b)pnga_release(g_b, _lo, _hi);
       }
 
     /*convert from C data type to ARMCI type */
@@ -629,7 +583,6 @@ Integer bndim, bdims[MAXDIM];
      }
    }
     
-   GA_POP_NAME;
 
 }
 
@@ -644,6 +597,10 @@ void pnga_scale(Integer g_a, void* alpha)
   Integer num_blocks;
   void *ptr;
   int local_sync_begin,local_sync_end;
+  Integer _dims[MAXDIM];
+  Integer _ld[MAXDIM-1];
+  Integer _lo[MAXDIM];
+  Integer _hi[MAXDIM];
 
   local_sync_begin = _ga_sync_begin; local_sync_end = _ga_sync_end;
   _ga_sync_begin = 1; _ga_sync_end=1; /*remove any previous masking*/
@@ -653,21 +610,20 @@ void pnga_scale(Integer g_a, void* alpha)
   me = pnga_pgroup_nodeid(grp_id);
 
   pnga_check_handle(g_a, "ga_scale");
-  GA_PUSH_NAME("ga_scale");
   num_blocks = pnga_total_blocks(g_a);
 
-  pnga_inquire(g_a, &type, &ndim, dims);
+  pnga_inquire(g_a, &type, &ndim, _dims);
   if (num_blocks < 0) {
-    pnga_distribution(g_a, me, lo, hi);
+    pnga_distribution(g_a, me, _lo, _hi);
     if (pnga_has_ghosts(g_a)) {
-      pnga_scale_patch(g_a, lo, hi, alpha);
+      pnga_scale_patch(g_a, _lo, _hi, alpha);
       return;
     }
 
-    if ( lo[0]> 0 ){ /* base index is 1: we get 0 if no elements stored on p */
+    if ( _lo[0]> 0 ){ /* base index is 1: we get 0 if no elements stored on p */
 
-      pnga_access_ptr(g_a, lo, hi, &ptr, ld);
-      GET_ELEMS(ndim,lo,hi,ld,&elems);
+      pnga_access_ptr(g_a, _lo, _hi, &ptr, _ld);
+      GET_ELEMS(ndim,_lo,_hi,_ld,&elems);
 
       switch (type){
         int *ia;
@@ -719,7 +675,7 @@ void pnga_scale(Integer g_a, void* alpha)
       }
 
       /* release access to the data */
-      pnga_release_update(g_a, lo, hi);
+      pnga_release_update(g_a, _lo, _hi);
     }
   } else {
     pnga_access_block_segment_ptr(g_a, me, &ptr, &elems);
@@ -774,7 +730,6 @@ void pnga_scale(Integer g_a, void* alpha)
     /* release access to the data */
     pnga_release_update_block_segment(g_a, me);
   }
-  GA_POP_NAME;
   if(local_sync_end)pnga_pgroup_sync(grp_id); 
 }
 
@@ -789,7 +744,11 @@ register Integer i;
 void *ptr_a, *ptr_b, *ptr_c;
 Integer a_grp, b_grp, c_grp;
 int local_sync_begin,local_sync_end;
-
+ 
+ Integer _dims[MAXDIM];
+ Integer _ld[MAXDIM-1];
+ Integer _lo[MAXDIM];
+ Integer _hi[MAXDIM];
  Integer andim, adims[MAXDIM];
  Integer bndim, bdims[MAXDIM];
  Integer cndim, cdims[MAXDIM];
@@ -797,7 +756,6 @@ int local_sync_begin,local_sync_end;
    local_sync_begin = _ga_sync_begin; local_sync_end = _ga_sync_end;
    _ga_sync_begin = 1; _ga_sync_end=1; /*remove any previous masking*/
 
-   GA_PUSH_NAME("ga_add");
    a_grp = pnga_get_pgroup(g_a);
    b_grp = pnga_get_pgroup(g_b);
    c_grp = pnga_get_pgroup(g_c);
@@ -820,28 +778,27 @@ int local_sync_begin,local_sync_end;
        pnga_add_patch(alpha, g_a, one_arr, adims, beta, g_b, one_arr, bdims,
                       g_c, one_arr, cdims);
        
-       GA_POP_NAME;
        return;
    }
 
    pnga_pgroup_sync(a_grp);
-   pnga_inquire(g_c,  &typeC, &ndim, dims);
-   pnga_distribution(g_c, me, lo, hi);
-   if (  lo[0]>0 ){
-     pnga_access_ptr(g_c, lo, hi, &ptr_c, ld);
-     GET_ELEMS(ndim,lo,hi,ld,&elems);
+   pnga_inquire(g_c,  &typeC, &ndim, _dims);
+   pnga_distribution(g_c, me, _lo, _hi);
+   if (  _lo[0]>0 ){
+     pnga_access_ptr(g_c, _lo, _hi, &ptr_c, _ld);
+     GET_ELEMS(ndim,_lo,_hi,_ld,&elems);
    }
 
    if(g_a == g_c){
      ptr_a  = ptr_c;
      elemsa = elems;
    }else { 
-     pnga_inquire(g_a,  &type, &ndim, dims);
+     pnga_inquire(g_a,  &type, &ndim, _dims);
      if(type != typeC) pnga_error("types not consistent", g_a);
-     pnga_distribution(g_a, me, lo, hi);
-     if (  lo[0]>0 ){
-       pnga_access_ptr(g_a, lo, hi, &ptr_a, ld);
-       GET_ELEMS(ndim,lo,hi,ld,&elemsa);
+     pnga_distribution(g_a, me, _lo, _hi);
+     if (  _lo[0]>0 ){
+       pnga_access_ptr(g_a, _lo, _hi, &ptr_a, _ld);
+       GET_ELEMS(ndim,_lo,_hi,_ld,&elemsa);
      }
    }
 
@@ -849,19 +806,19 @@ int local_sync_begin,local_sync_end;
      ptr_b  = ptr_c;
      elemsb = elems;
    }else {
-     pnga_inquire(g_b,  &type, &ndim, dims);
+     pnga_inquire(g_b,  &type, &ndim, _dims);
      if(type != typeC) pnga_error("types not consistent", g_b);
-     pnga_distribution(g_b, me, lo, hi);
-     if (  lo[0]>0 ){
-       pnga_access_ptr(g_b, lo, hi, &ptr_b, ld);
-       GET_ELEMS(ndim,lo,hi,ld,&elemsb);
+     pnga_distribution(g_b, me, _lo, _hi);
+     if (  _lo[0]>0 ){
+       pnga_access_ptr(g_b, _lo, _hi, &ptr_b, _ld);
+       GET_ELEMS(ndim,_lo,_hi,_ld,&elemsb);
      }
    }
 
    if(elems!= elemsb)pnga_error("inconsistent number of elements a",elems-elemsb);
    if(elems!= elemsa)pnga_error("inconsistent number of elements b",elems-elemsa);
 
-   if (  lo[0]>0 ){
+   if (  _lo[0]>0 ){
 
        /* operation on the "local" piece of data */
        switch(type){
@@ -938,13 +895,12 @@ int local_sync_begin,local_sync_end;
        }
 
        /* release access to the data */
-       pnga_release_update(g_c, lo, hi);
-       if(g_c != g_a)pnga_release(g_a, lo, hi);
-       if(g_c != g_b)pnga_release(g_b, lo, hi);
+       pnga_release_update(g_c, _lo, _hi);
+       if(g_c != g_a)pnga_release(g_a, _lo, _hi);
+       if(g_c != g_b)pnga_release(g_b, _lo, _hi);
    }
 
 
-   GA_POP_NAME;
    if(local_sync_end)pnga_pgroup_sync(a_grp);
 }
 
@@ -952,7 +908,7 @@ int local_sync_begin,local_sync_end;
 static 
 void snga_local_transpose(Integer type, char *ptra, Integer n, Integer stride, char *ptrb)
 {
-int i;
+Integer i;
     switch(type){
 
        case C_INT:
@@ -994,14 +950,15 @@ int i;
 void pnga_transpose(Integer g_a, Integer g_b)
 {
 Integer me = pnga_nodeid();
+Integer _ld[MAXDIM-1];
 Integer nproc = pnga_nnodes(); 
 Integer atype, btype, andim, adims[MAXDIM], bndim, bdims[MAXDIM];
-Integer lo[2],hi[2];
+Integer lo[2],hi[2],ld[2];
 int local_sync_begin,local_sync_end;
 Integer num_blocks_a;
 char *ptr_tmp, *ptr_a;
+_iterator_hdl hdl;
 
-    GA_PUSH_NAME("ga_transpose");
     
     local_sync_begin = _ga_sync_begin; local_sync_end = _ga_sync_end;
     _ga_sync_begin = 1; _ga_sync_end=1; /*remove any previous masking*/
@@ -1015,6 +972,32 @@ char *ptr_tmp, *ptr_a;
     if(bndim != 2 || andim != 2) pnga_error("dimension must be 2",0);
     if(atype != btype ) pnga_error("array type mismatch ", 0L);
 
+#if 1
+    pnga_local_iterator_init(g_a, &hdl);
+    while (pnga_local_iterator_next(&hdl,lo,hi,&ptr_a,ld)) {
+      Integer idx;
+      Integer block_dims[MAXDIM];
+      Integer nelem, lob[2], hib[2], nrow, ncol;
+      int i, size=GAsizeofM(atype);
+
+      nelem = (hi[0]-lo[0]+1)*(hi[1]-lo[1]+1);
+      ptr_tmp = (char *) ga_malloc(nelem, atype, "transpose_tmp");
+
+      nrow   = hi[0] -lo[0]+1;
+      ncol   = hi[1] -lo[1]+1; 
+      nelem  = nrow*ncol;
+      lob[0] = lo[1]; lob[1] = lo[0];
+      hib[0] = hi[1]; hib[1] = hi[0];
+      for(i = 0; i < ncol; i++){
+        char *ptr = ptr_tmp + i*size;
+
+        snga_local_transpose(atype, ptr_a, nrow, ncol*size, ptr);
+        ptr_a += ld[0]*size;
+      }
+      pnga_put(g_b, lob, hib, ptr_tmp ,&ncol);
+      ga_free(ptr_tmp);
+    }
+#else
     num_blocks_a = pnga_total_blocks(g_a);
 
     if (num_blocks_a < 0) {
@@ -1034,13 +1017,13 @@ char *ptr_tmp, *ptr_a;
         ptr_tmp = (char *) ga_malloc(nelem, atype, "transpose_tmp");
 
         /* get access to local data */
-        pnga_access_ptr(g_a, lo, hi, &ptr_a, ld);
+        pnga_access_ptr(g_a, lo, hi, &ptr_a, _ld);
 
         for(i = 0; i < ncol; i++){
           char *ptr = ptr_tmp + i*size;
 
           snga_local_transpose(atype, ptr_a, nrow, ncol*size, ptr);
-          ptr_a += ld[0]*size;
+          ptr_a += _ld[0]*size;
         }
 
         pnga_release(g_a, lo, hi); 
@@ -1064,7 +1047,7 @@ char *ptr_tmp, *ptr_a;
       if (!pnga_uses_proc_grid(g_a)) {
         for (idx = me; idx < num_blocks_a; idx += nproc) {
           pnga_distribution(g_a, idx, lo, hi);
-          pnga_access_block_ptr(g_a, idx, &ptr_a, ld);
+          pnga_access_block_ptr(g_a, idx, &ptr_a, _ld);
 
           nrow   = hi[0] -lo[0]+1;
           ncol   = hi[1] -lo[1]+1; 
@@ -1075,7 +1058,7 @@ char *ptr_tmp, *ptr_a;
             char *ptr = ptr_tmp + i*size;
 
             snga_local_transpose(atype, ptr_a, nrow, ncol*size, ptr);
-            ptr_a += ld[0]*size;
+            ptr_a += _ld[0]*size;
           }
           pnga_put(g_b, lob, hib, ptr_tmp ,&ncol);
 
@@ -1100,7 +1083,7 @@ char *ptr_tmp, *ptr_a;
         }
 
         if (ichk) {
-          pnga_access_block_grid_ptr(g_a, index, &ptr_a, ld);
+          pnga_access_block_grid_ptr(g_a, index, &ptr_a, _ld);
           while (index[andim-1] < blocks[andim-1]) {
             /* find bounding coordinates of block */
             chk = 1;
@@ -1111,7 +1094,7 @@ char *ptr_tmp, *ptr_a;
               if (hi[i] < lo[i]) chk = 0;
             }
             if (chk) {
-              pnga_access_block_grid_ptr(g_a, index, &ptr_a, ld);
+              pnga_access_block_grid_ptr(g_a, index, &ptr_a, _ld);
               nrow   = hi[0] -lo[0]+1;
               ncol   = hi[1] -lo[1]+1; 
               nelem  = nrow*ncol;
@@ -1120,7 +1103,7 @@ char *ptr_tmp, *ptr_a;
               for(i = 0; i < ncol; i++){
                 char *ptr = ptr_tmp + i*size;
                 snga_local_transpose(atype, ptr_a, nrow, block_dims[0]*size, ptr);
-                ptr_a += ld[0]*size;
+                ptr_a += _ld[0]*size;
               }
               pnga_put(g_b, lob, hib, ptr_tmp ,&block_dims[0]);
               pnga_release_update_block_grid(g_a, index);
@@ -1138,7 +1121,6 @@ char *ptr_tmp, *ptr_a;
       }
       ga_free(ptr_tmp);
     }
-
+#endif
     if(local_sync_end)pnga_sync();
-    GA_POP_NAME;
 }
